#include <pcap.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#define SNAP_LEN 1518    // 以太网帧最大长度
#define SIZE_ETHERNET 14 // 以太网包头长度 mac 6*2, type: 2
#define ETHER_ADDR_LEN 6 // mac地址长度

struct packet_ethernet
{
    u_char ether_dhost[ETHER_ADDR_LEN]; /* destination host address */
    u_char ether_shost[ETHER_ADDR_LEN]; /* source host address */
    u_short ether_type;                 /* IP? ARP? RARP? etc */
};

/* IP header */
struct packet_ip
{
    u_char ip_vhl;                 /* version << 4 | header length >> 2 */
    u_char ip_tos;                 /* type of service */
    u_short ip_len;                /* total length */
    u_short ip_id;                 /* identification */
    u_short ip_off;                /* fragment offset field */
#define IP_RF 0x8000               /* reserved fragment flag */
#define IP_DF 0x4000               /* dont fragment flag */
#define IP_MF 0x2000               /* more fragments flag */
#define IP_OFFMASK 0x1fff          /* mask for fragmenting bits */
    u_char ip_ttl;                 /* time to live */
    u_char ip_p;                   /* protocol */
    u_short ip_sum;                /* checksum */
    struct in_addr ip_src, ip_dst; /* source and dest address */
    // struct in_addr ip_src;
    // struct in_addr ip_dst;  		    /* source and dest address */
};
#define IP_HL(ip) (((ip)->ip_vhl) & 0x0f)
#define IP_V(ip) (((ip)->ip_vhl) >> 4)

/* TCP header */
typedef u_int tcp_seq;

struct packet_tcp
{
    u_short th_sport; /* source port */
    u_short th_dport; /* destination port */
    tcp_seq th_seq;   /* sequence number */
    tcp_seq th_ack;   /* acknowledgement number */
    u_char th_offx2;  /* data offset, rsvd */
#define TH_OFF(th) (((th)->th_offx2 & 0xf0) >> 4)
    u_char th_flags;
#define TH_FIN 0x01
#define TH_SYN 0x02
#define TH_RST 0x04
#define TH_PUSH 0x08
#define TH_ACK 0x10
#define TH_URG 0x20
#define TH_ECE 0x40
#define TH_CWR 0x80
#define TH_FLAGS (TH_FIN | TH_SYN | TH_RST | TH_ACK | TH_URG | TH_ECE | TH_CWR)
    u_short th_win; /* window */
    u_short th_sum; /* checksum */
    u_short th_urp; /* urgent pointer */
};

/* 以太网数据帧内容解析，满足特定网络通讯要求*/
void loop_callback(u_char *args, const struct pcap_pkthdr *header, const u_char *packet)
{
    static int count = 0;                   // 包计数器
    const struct packet_ethernet *ethernet; /* The ethernet header [1] */
    const char *payload;                    /* Packet payload */

    int size_ip;
    int size_tcp;
    int size_payload;
    unsigned short frame_type;

    count++;

    /* 以太网头 */
    ethernet = (struct packet_ethernet *)(packet);

    if (ethernet->ether_type == 0x0081)
    {
        printf("\ndst %02x:%02x:%02x:%02x:%02x:%02x src %02x:%02x:%02x:%02x:%02x:%02x type=%04x\n",
            ethernet->ether_dhost[0],
            ethernet->ether_dhost[1],
            ethernet->ether_dhost[2],
            ethernet->ether_dhost[3],
            ethernet->ether_dhost[4],
            ethernet->ether_dhost[5],
            ethernet->ether_shost[0],
            ethernet->ether_shost[1],
            ethernet->ether_shost[2],
            ethernet->ether_shost[3],
            ethernet->ether_shost[4],
            ethernet->ether_shost[5],
            ethernet->ether_type);
        // 内容
        payload = (u_char *)(packet + SIZE_ETHERNET);
        frame_type = * ((unsigned short *)payload+1);
        printf("frame_type = %04x\n",frame_type);
        if(frame_type == 0xBA88)    //希望的数据帧
        {
            // 内容长度
            size_payload = header->len - 18;
            printf("%lu %lu len=%d :\n",header->ts.tv_sec, header->ts.tv_usec, size_payload);

            if (size_payload > 0)
            {
                for(int i=0;i<size_payload;i++)
                {
                    printf("%02x ", *(payload + 4 + i));
                    if(i!=0 && (i%16 == 0))
                    {
                        printf("\n");
                    }
                }
                // printf("%d %d %d %d bytes\n", (tcp->th_seq), ntohs(tcp->th_ack), ntohs(tcp->th_flags), size_payload);
                // write(payload, size_payload);
            }
            else
            {
                printf("%d \n", ntohs(TH_SYN));
            }
        }
    }
}

int main(int argc, char *argv[])
{
    pcap_t *handle;                                    /* 会话句柄 */
    char *dev;                                         /* 执行嗅探的设备 */
    char errbuf[PCAP_ERRBUF_SIZE];                     /* 存储错误信息的字符串 */
    struct bpf_program filter;                         /* 已经编译好的过滤器 */
    char filter_app[] = "ether dst aa:cb:7d:a6:e7:6d "; /* 过滤表达式 */
    bpf_u_int32 mask;                                  /* 所在网络的掩码 */
    bpf_u_int32 net;                                   /* 主机的IP地址 */
    struct pcap_pkthdr header;                         /* 由pcap.h定义 */
    const u_char *packet;                              /* 实际的包 */
    
    /* Define the device */
    /* dev = pcap_lookupdev(errbuf); */
    dev = "eth0"; /* 网卡名称 */
                  // dev = pcap_lookupdev(errbuf);	// 获取网络接口
                  // printf("dev=%s\n",dev);
    /* 探查设备属性 */
    if (pcap_lookupnet(dev, &net, &mask, errbuf) == -1)
    {
        fprintf(stderr, "couldn't get netmask for device %s: %s\n", dev, errbuf);
        net = 0;
        mask = 0;
    }

    handle = pcap_open_live(dev, 65536, 1, 1000, errbuf); /* 以混杂模式打开会话 */
    if (NULL == handle)
    {
        printf("%s", (char *)errbuf);
        exit(-1);
    }
    /* 编译并应用过滤器 */
    if (pcap_compile(handle, &filter, filter_app, 0, mask) == -1)
    {
        fprintf(stderr, "couldn't parse filter %s: %s\n", filter_app, pcap_geterr(handle));
        return (2);
    }
    if (pcap_setfilter(handle, &filter) == -1)
    {
        fprintf(stderr, "couldn't install filter %s: %s\n", filter_app, pcap_geterr(handle));
        return (2);
    }

    pcap_loop(handle, 0, loop_callback, NULL);
    pcap_close(handle); /* 关闭会话 */
    return (0);
}
